//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2019
//#
//#
//# File Name:  EPT_10M04_AF_S2_Top.v
//# Author:     Earth People Technology
//# Date:       June 10, 2019
//# Revision:   A
//#
//# Development: EPT Data Collection Project 
//# Application: Altera MAX10 FPGA
//# Description: This file contains verilog code which access
//#              the I/O's of the FPGA. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			6/10/19 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_10M04_AF_S2_Top (

	
	input  wire                CLK_50MHZ,
	input  wire                CLK_32KHZ,
	input  wire                RST,
	
	output wire                CLK_50MHZ_ENABLE,
	output wire                CLK_32KHZ_ENABLE,
	
	//UART COMMAND BUS
	output wire                UART_OUT, //To external 
    input wire                 UART_IN, //From external
  
	//I2C Bus 0
    inout  wire                I2C_SDA_0, 
    output wire                I2C_SCL_0,
	
	output wire   [7:0]        XIO_1,       //
	input  wire   [7:0]        XIO_2,       //
    input  wire   [7:0]        XIO_3,       //
	output wire   [8:0]        XIO_4,       //
	
	
	output reg     [7:0]       LED
	);

   //-----------------------------------------------
   // Parameters
   //-----------------------------------------------
   
   //LED State Machine Parameters
   parameter                   IDLE = 0,
	                           SELECT_MODE = 1,
                               LOAD_REGISTER = 2,
                               LOAD_LEDS = 3,
                               WAIT_FOR_TIMER = 6;

   parameter                   GLOBAL_RESET_COUNT = 12'h9c8;
   
   //Control Register selections for the LED Blinky
   parameter                   RANDOM_NUMBER = 4'h1;
   parameter                   SHIFT_LEFT = 4'h2;
   parameter                   SHIFT_RIGHT = 4'h3;
   parameter                   COUNT_UP = 8'h8;
   parameter                   COUNT_DOWN = 8'h8;
   parameter                   STATIC_VALUE = 6'h6;
   
   //Timer Value Low Limit
   parameter                   TIMER_LOW_LIMIT = 24'hf9c18;
   
   //MEMORY_TRANSFER_BLOCK_SIZE
   parameter                   MEMORY_TRANSFER_BLOCK_SIZE = 8'hff;
   
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************

	//Finite State Machine control registers
	reg 	[6:0]              state, next;
	
	//Active Transfer Registers
	wire [1:0]                 aa;

	//LED Blinky registers
	reg [23:0]                 timer_value;	
	reg [7:0]                  seed_value;
	reg [23:0]                 led_delay_counter;
	reg                        start_blinky;
	reg [7:0]                  static_load_value;
	reg [7:0]                  led_reg;
	wire                       led_reset;
	reg [7:0]                  shift_count_value;

	//UC Bus 
	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;

	//Trigger Signals
	wire [7:0]                  trigger_out;
	wire [7:0]                  trigger_in_byte;	
	
	//Transfer registers
	wire                        transfer_1_start;
	wire                        transfer_receive_busy;
	wire                        transfer_1_receive_in;
	wire [7:0]                  transfer_1_address;
	reg  [2:0]                  block_transfer_state;

	reg                         transfer_2_start;
	wire                        transfer_2_receive_in;
	wire   [7:0]                transfer_2_data_in;
	wire   [7:0]                transfer_2_data_out;
	wire   [7:0]                transfer_2_data;
	
	reg                         transfer_3_start;
	wire                        transfer_3_receive_in;
	wire   [7:0]                transfer_3_control_register_number;
	
	reg                         transfer_4_start;
	wire                        transfer_4_receive_in;
	wire   [7:0]                transfer_4_control_register_contents;

	//Serial SPI Receive Registers Unit 1
	wire  [7:0]                 serial_receive_out_byte;
	reg                         master_spi_initiate_receive;
	wire                        spi_byte_received;
	
	//Receive Storage registers Unit 1
    reg  [1:0]                  samples_per_block_state;

	//Serial SPI Transmit Registers
	wire                        spi_transmit_complete;
	
	//channel_selected_transmit_register
	reg   [7:0]                 channel_selected_transmit_register;	
	reg                         channel_selected_spi_transmit_start;
	
	//SPI Command registers
	reg   [7:0]                 spi_command_register;
	reg                         spi_command_start; 
	
	//SPI Command Multiplexor
	reg   [1:0]                 spi_write_control_reg;
	reg   [1:0]                 spi_read_control_reg;
	reg   [1:0]                 channel_selected_spi_transmit_start_reg;
		
	//I2C Bus 0 registers
	reg   [7:0]                 i2c_0_command_register;
	reg                         i2c_0_command_start;
	reg                         i2c_0_command_start_reg;
	wire                        i2c_serial_clk;
	wire                        i2c_0_transmit_complete;
	wire                        i2c_0_local_memory_wr_latch;
	wire                        i2c_0_local_memory_oe;
	wire                        i2c_0_local_memory_rd_latch;
	wire                        i2c_0_byte_received;
	
	//Block 1 Control Registers
	reg                         start_block_out;
	reg                         start_block_out_reg;
	wire                        block_byte_ready;
	wire                        block_in_rcv;
	wire  [7:0]                 uc_length;
	wire                        block_busy;
	wire  [7:0]                 block_out_byte;
	reg   [15:0]                block_transfer_mem_array_selection; 
	reg   [7:0]                 block_transfer_mem_array_count;


	
	
	//Timer Tick Main Counter
	reg   [18:0]                 timer_tick_counter;
	
	//Host Interface registers
	reg                         host_oe;
	reg                         host_rd_latch;
	reg                         host_wr_latch;
	reg   [3:0]                 host_rd_latch_state;
	reg   [3:0]                 host_wr_latch_state;
	
	//Memory Interface registers
	wire   [7:0]                local_memory_address;
	wire   [7:0]                local_memory_data;
	wire                        local_memory_wr_latch;
	wire                        local_memory_rd_latch;
	wire                        local_memory_oe;
	wire                        adc_convst_delay_local_memory_wr_latch;
	wire                        spi_local_memory_wr_latch;
	wire                        adc_convst_delay_local_memory_rd_latch;
	wire                        spi_local_memory_rd_latch;
	wire                        adc_convst_delay_local_memory_oe;
	wire                        spi_local_memory_oe;
	
	//UC Control Register
	wire   [7:0]                adc_control_reg;	
	wire   [7:0]                spi_control_reg;
	wire   [7:0]                i2c_0_control_reg;
	wire   [7:0]                i2c_1_control_reg;
	wire   [7:0]                adc_convst_control_reg;
	wire   [7:0]                reset_control_reg;
	wire                        request_local_memory;
	wire                        adc_convst_delay_request_memory;
	wire                        spi_comms_request_memory;	
	wire                        grant_local_memory;
	wire                        i2c_0_comms_request_memory;
	
	//Reset Control
	reg                         reset_reg;


	//TEST ONLY
	wire                        transfer_busy;
	reg   [3:0]                 STATE_OUT;
	wire  [3:0]                 uc_controller_memory_state;

	
	
`ifdef SIM
   reg [8*26:1] state_name;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************

   //Clock and Reset
   assign            aa[1] = CLK_50MHZ;
   assign            aa[0] = RST;
   //assign            reset = reset_signal_reg;
   
   //Clock Enables
   assign            CLK_50MHZ_ENABLE  = 1'b1;
   assign            CLK_32KHZ_ENABLE = 1'b0;
   
   
   //Control Register Map
   assign            led_reset = XIO_2[0];
   
   //Local Memory Request/Grant registers
  // assign            request_local_memory = adc_convst_delay_request_memory | spi_comms_request_memory | i2c_0_comms_request_memory;
   assign            request_local_memory = i2c_0_comms_request_memory;
   
   //Host Interface data bus
   assign             transfer_2_data = !host_oe ? transfer_2_data_in : 8'hzz;
   assign             transfer_2_data_out = host_oe ? transfer_2_data : 8'h00;
   
   //Local Memory Write Latch register
   assign             local_memory_wr_latch = /*spi_local_memory_wr_latch | adc_convst_delay_local_memory_wr_latch |*/ i2c_0_local_memory_wr_latch;
   assign             local_memory_oe = /*spi_local_memory_oe | adc_convst_delay_local_memory_oe |*/ i2c_0_local_memory_oe;
   assign             local_memory_rd_latch = /*spi_local_memory_rd_latch | adc_convst_delay_local_memory_rd_latch |*/ i2c_0_local_memory_rd_latch;
 
   //Test
   assign              XIO_1[0] = 1'b0;
   assign              XIO_1[1] = 1'b0;
   assign              XIO_1[2] = 1'b0;
   assign              XIO_1[3] = 1'b0;
   assign              XIO_1[4] = 1'b1;
   assign              XIO_1[5] = 1'b0;
   assign              XIO_1[6] = 1'b0;
   assign              XIO_1[7] = 1'b0;
   
   assign              XIO_4[8] = 1'b1;
   assign              XIO_4[7] = 1'b0;
   assign              XIO_4[6] = 1'b0;
   assign              XIO_4[5] = 1'b0;
   assign              XIO_4[4] = 1'b0;
   assign              XIO_4[3] = 1'b0;
   assign              XIO_4[2] = 1'b0;
   assign              XIO_4[1] = 1'b0;
   assign              XIO_4[0] = 1'b0;
   
   //assign            LED[0] = 1'b0;
   //assign            LED[1] = 1'b1;
   //assign            LED[2] = 1'b0;
   //assign            LED[3] = 1'b1;
   
   
//**************************************************
//* 	Reset Signal	
//**************************************************
/*	always @(posedge CLK_50MHZ or negedge RST)
	begin
	  if(!RST)
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end
*/
   //-----------------------------------------------
   // Load Timer Value
   //-----------------------------------------------

  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     timer_value <= TIMER_LOW_LIMIT;
	else 
	begin
	     //if(XIO_2[4])
	     //    timer_value[15:8] <= 8'h00;
	     //else if(XIO_2[5])
	     //    timer_value[23:16] <= 8'h80;
	end 
  end

   //-----------------------------------------------
   // LED Blinky start 
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
         start_blinky <= 1'b1;
	else 
	begin
	     start_blinky <= 1'b1;
	    //if(XIO_1[0] == 1'b1)
        //    start_blinky <= 1'b1;
		//else
        //    start_blinky <= 1'b0;		
 	end 
  end
 
   //-----------------------------------------------
   // LED Delay Timer Counter 
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     led_delay_counter <= TIMER_LOW_LIMIT;
	else 
	begin
	     if(state[SELECT_MODE])
		 led_delay_counter <= timer_value;
		 else if(state[WAIT_FOR_TIMER])
	         led_delay_counter <= led_delay_counter - 1'd1;
	end 
  end

   
   //-----------------------------------------------
   // Select the LED Function from the control
   // register bits 7 to 4   
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     led_reg <= 8'h1;
	else 
	begin
        if (state[SELECT_MODE])
	    begin
	         //case(XIO_2[7:4])
		     //  SHIFT_LEFT:
		          led_reg = shift_count_value;
		     //  SHIFT_RIGHT:
		     //     led_reg = shift_count_value;
		     //  COUNT_UP:
		     //     led_reg = shift_count_value;
		     //  COUNT_DOWN:
		     //     led_reg = shift_count_value;
		     //  STATIC_VALUE:
		    //      led_reg = static_load_value;
		    //   default:
		    //      led_reg = shift_count_value;
		    //  endcase 
        end	
    end
  end
  
   //-----------------------------------------------
   // Increment the Shift/Count value from the control
   // register bits 7 to 4   
   //-----------------------------------------------
  always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     shift_count_value <= 4'h1;
	else 
	begin
		if (state[LOAD_REGISTER])
	    begin
	         //case(XIO_4[7:4])
		     //SHIFT_LEFT:
			 //begin
		        shift_count_value[0] <= led_reg[7];
		        shift_count_value[7] <= led_reg[6];
		        shift_count_value[6] <= led_reg[5];
		        shift_count_value[5] <= led_reg[4];
		        shift_count_value[4] <= led_reg[3];
		        shift_count_value[3] <= led_reg[2];
		        shift_count_value[2] <= led_reg[1];
		        shift_count_value[1] <= led_reg[0];
		    // end
		    // SHIFT_RIGHT:
			// begin
		    //    shift_count_value[0] <= led_reg[1];
		    //    shift_count_value[3] <= led_reg[0];
		    //    shift_count_value[2] <= led_reg[3];
		    //    shift_count_value[1] <= led_reg[2];
		    // end
		    // COUNT_UP:
		    //    shift_count_value <= led_reg + 1'd1;
		    // COUNT_DOWN:
		    //    shift_count_value <= led_reg - 1'd1;
		    // default:
			// begin
		    //    shift_count_value[0] <= led_reg[3];
		    //    shift_count_value[3] <= led_reg[2];
		    //    shift_count_value[2] <= led_reg[1];
		    //    shift_count_value[1] <= led_reg[0];
		    // end
		    //endcase   
         end
	end 
  end

   //-----------------------------------------------
   // Set the LED outputs 
   //-----------------------------------------------
   always @(posedge CLK_50MHZ or negedge RST)
  begin
    if(!RST)
	     LED <= 8'hz;
	else 
	begin
	  if(state[LOAD_LEDS])
	  begin
		if ( led_reg[0] )
			LED[0] = 1'b0;
		else 
			LED[0] = 1'bz;
			
		if ( led_reg[1] )
			LED[1] = 1'b0;
		else 
			LED[1] = 1'bz;
			
 		if ( led_reg[2] )
			LED[2] = 1'b0;
		else 
			LED[2] = 1'bz;
			
		if ( led_reg[3] )
			LED[3] = 1'b0;
		else 
			LED[3] = 1'bz;
			
		if ( led_reg[4] )
			LED[4] = 1'b0;
		else 
			LED[4] = 1'bz;
			
		if ( led_reg[5] )
			LED[5] = 1'b0;
		else 
			LED[5] = 1'bz;
			
 		if ( led_reg[6] )
			LED[6] = 1'b0;
		else 
			LED[6] = 1'bz;
			
		if ( led_reg[7] )
			LED[7] = 1'b0;
		else 
			LED[7] = 1'bz;
	  end
	end
  end

   //-----------------------------------------------
   // Gain settings for the Analog Signal Conditioning 
   // circuit. Uses I2C Bus 0. Each channel address, 
   // command and data are written into uc_controller
   // memory then read by the I2C module.
   //-----------------------------------------------
	always @(posedge CLK_50MHZ or negedge RST)
	begin
	  if(!RST)
	  begin
            i2c_0_command_register <= 0;
			i2c_0_command_start <= 1'b0;
			i2c_0_command_start_reg <= 1'b0;
	  end
	  else
	  begin
	      if(i2c_0_control_reg[7] & !i2c_0_command_start_reg)
	      begin
		    i2c_0_command_register <= i2c_0_control_reg;
			i2c_0_command_start <= 1'b1;
			i2c_0_command_start_reg <= 1'b1;
	      end
	      else if(i2c_0_control_reg[7] & i2c_0_command_start_reg)
	      begin
			i2c_0_command_start <= 1'b0;
			i2c_0_command_start_reg <= 1'b1;
	      end
	      else if(!i2c_0_control_reg[7] & i2c_0_command_start_reg)
	      begin
			i2c_0_command_start <= 1'b0;
			i2c_0_command_start_reg <= 1'b0;
	      end
	  end
	end  

    //-----------------------------------------------
   // Timer Tick Main Counter.
   //-----------------------------------------------
	always @(posedge CLK_50MHZ or negedge aa[0])
	begin
	  if(!aa[0])
	  begin
			timer_tick_counter <= 0;
	  end
	  else
	  begin
	      if(trigger_in_byte)
	      begin
             timer_tick_counter <= 0;
          end
		  else
		  begin
			timer_tick_counter <= timer_tick_counter + 1'd1;
	      end
	  end
	end  

   //-----------------------------------------------
   // Read Latch and Output Enable register decode.
   //-----------------------------------------------
	always @(posedge CLK_50MHZ or negedge RST)
	begin
	  if(!RST)
	  begin
			host_rd_latch <= 1'b0;
			host_oe <= 1'b0;
			transfer_2_start <= 1'b0;
			host_rd_latch_state <= 2'b00;
	  end
	  else
	  begin
         case(host_rd_latch_state)
         2'b00:
         begin
	        if(trigger_in_byte[2])
			begin
			   host_oe <= 1'b1;
			   host_rd_latch_state <= 2'b01;
			end
         end
         2'b01:
         begin
	        if(transfer_1_receive_in)
	        begin
	           host_rd_latch <= 1'b0;
			   host_rd_latch_state <= 2'b10;
	        end
         end
         2'b10:
         begin
	        if(trigger_in_byte[1])
	        begin
	           host_rd_latch <= 1'b1;
			   transfer_2_start <= 1'b1;
			   host_rd_latch_state <= 2'b11;
	        end
         end
         2'b11:
         begin
		    if(timer_tick_counter[5])
	        begin
	           host_rd_latch <= 1'b0;
			   host_oe <= 1'b0;
			   transfer_2_start <= 1'b0;
			   host_rd_latch_state <= 2'b00;
	        end
         end
         endcase
	  end
	end  



   //-----------------------------------------------
   // Write Latch register decode.
   //-----------------------------------------------
	always @(posedge CLK_50MHZ or negedge RST)
	begin
	  if(!RST)
	  begin
			host_wr_latch <= 1'b0;
			host_wr_latch_state <= 2'b00;
	  end
	  else
	  begin
         case(host_wr_latch_state)
         2'b00:
         begin
	        if(transfer_1_receive_in & !host_oe)
	        begin
	           host_wr_latch <= 1'b0;
			   host_wr_latch_state <= 2'b01;
	        end
         end
         2'b01:
         begin
	        if(transfer_2_receive_in)
	        begin
	           host_wr_latch <= 1'b0;
			   host_wr_latch_state <= 2'b10;
	        end
         end
         2'b10:
         begin
	        if(trigger_in_byte[0])
	        begin
	           host_wr_latch <= 1'b1;
			   host_wr_latch_state <= 2'b11;
	        end
         end
         2'b11:
         begin
		    if(timer_tick_counter[5])
	        begin
	           host_wr_latch <= 1'b0;
			   host_wr_latch_state <= 2'b00;
 	        end
        end
         endcase
	  end
	end  

 
    //-----------------------------------------------
   // State Machine: Send Pattern to LEDs 
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_50MHZ or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 7'h00;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or  start_blinky or
		      led_reg or led_delay_counter or 
			  timer_value or shift_count_value) 
     begin
	     next = 7'h00;

	if (state[IDLE])
	  begin
	     if (start_blinky)
	       next[SELECT_MODE] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[SELECT_MODE])
	begin
           next[LOAD_REGISTER] = 1'b1;
	end

	if (state[LOAD_REGISTER])
	begin
        next[LOAD_LEDS] = 1'b1;
    end
		
	if (state[LOAD_LEDS])
	begin
        next[WAIT_FOR_TIMER] = 1'b1;
    end
	
	if (state[WAIT_FOR_TIMER])
		if (led_delay_counter <= timer_value)
			next[WAIT_FOR_TIMER] = 1'b1;
		else 
			next[SELECT_MODE] = 1'b1;

			
`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << SELECT_MODE ))
		   state_name = "SELECT_MODE";
	   else if ( state == ( 1 << LOAD_REGISTER ))
		   state_name = "LOAD_REGISTER";
	   else if ( state == ( 1 << LOAD_LEDS ))
		   state_name = "LOAD_LEDS";
	   else if ( state == ( 1 << WAIT_FOR_TIMER ))
		   state_name = "WAIT_FOR_TIMER";
`endif


	end//end of state machine

   //-----------------------------------------------
   // I2C Slave SERDES Module 
   //-----------------------------------------------
    i2c_comms_interface         I2C_COMMS_INTERFACE_0_INST
	(
	.CLK                        (CLK_50MHZ),
	.RST_N                      (RST),
	
	.SERIAL_CLK                 (i2c_serial_clk),

	.I2C_DATA                   (I2C_SDA_0),
	.I2C_CLOCK                  (I2C_SCL_0),

	.MEMORY_ADDRESS             (local_memory_address),
	.MEMORY_DATA                (local_memory_data),
	.MEMORY_WRITE_LATCH         (i2c_0_local_memory_wr_latch),
	.MEMORY_READ_LATCH          (i2c_0_local_memory_rd_latch),
	.MEMORY_OUTPUT_ENABLE       (i2c_0_local_memory_oe),
	
	.I2C_COMMAND                (i2c_0_command_register),
	.I2C_COMMAND_LATCH          (i2c_0_command_start),
	
	.I2C_TRANSMIT_COMPLETE      (i2c_0_transmit_complete),
	.I2C_BYTE_RECEIVED          (i2c_0_byte_received),
	
	.REQUEST_EN                 (i2c_0_comms_request_memory),
	.GRANT_RDY                  (grant_local_memory)

     );


   //-----------------------------------------------
   // Instantiate the UC Controller
   //-----------------------------------------------
	uc_controller                               UC_CONTROLLER_INST
	(
	.CLK                                        (CLK_50MHZ), 
	.RST_N                                      (RST),

	.HOST_INTERFACE_ADDRESS                     (transfer_1_address),
	.HOST_INTERFACE_DATA                        (transfer_2_data),
	.HOST_INTERFACE_WRITE_LATCH                 (host_wr_latch),
	.HOST_INTERFACE_READ_LATCH                  (host_rd_latch),
	.HOST_INTERFACE_OUTPUT_ENABLE               (host_oe),

	.HOST_CONTROL_REG_NUMBER                    (transfer_3_control_register_number),
	.HOST_CONTROL_REG_CONTENTS                  (transfer_4_control_register_contents),
	.HOST_CONTROL_REG_NUMBER_LATCH              (transfer_3_receive_in),
	.HOST_CONTROL_REG_CONTENTS_LATCH            (transfer_4_receive_in),

	.CR_0                                       (adc_control_reg),
	.CR_1                                       (spi_control_reg),
	.CR_2                                       (i2c_0_control_reg),
	.CR_3                                       (i2c_1_control_reg),
	.CR_4                                       (adc_convst_control_reg),
	.CR_5                                       (reset_control_reg),
	.CR_6                                       (),
	.CR_7                                       (),
	
	.MEMORY_INTERFACE_ADDRESS                   (local_memory_address),
	.MEMORY_INTERFACE_DATA                      (local_memory_data),
	.MEMORY_INTERFACE_WRITE_LATCH               (local_memory_wr_latch),
	//.MEMORY_INTERFACE_WRITE_LATCH               (1'b0),
	.MEMORY_INTERFACE_READ_LATCH                (local_memory_rd_latch),
	.MEMORY_INTERFACE_OUTPUT_ENABLE             (local_memory_oe),

	.MEMORY_INTERFACE_EN                        (request_local_memory),
	.MEMORY_INTERFACE_RDY                       (grant_local_memory),
	
	.TIMEOUT_TICK                               (timer_tick_counter),

	.ERROR_OUT                                  (),

	.STATE_OUT                                  (uc_controller_memory_state)
	);

   //-----------------------------------------------
   // Instantiate the Serial Clock
   //-----------------------------------------------
	serial_clock               SERIAL_CLOCK_INST
	(	
	.CLK                       (CLK_50MHZ),
	.RST_N                     (RST),
	
	.SERIAL_CLK                (i2c_serial_clk)
	);


	
   //-----------------------------------------------
   // Instantiate the EPT Library
   //-----------------------------------------------

	active_serial_library	   ACTIVE_SERIAL_LIBRARY_INST
	(	
	.aa                        (aa),
	.UART_OUT                  (UART_OUT),
	.UART_IN                   (UART_IN),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT)//,
	
	//.STATE_OUT                 (ft245_test_state_out),
	//.TEST_OUT                  (active_transfer_test_out)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Active Modules
   //-----------------------------------------------
wire [22*6-1:0]  uc_out_m;
eptWireOR # (.N(6)) wireOR (UC_OUT, uc_out_m);

    //
	// Trigger_In Mapping
	// trigger_in_byte[0] = 
	// trigger_in_byte[1] = 
	// trigger_in_byte[2] = 
	// trigger_in_byte[3] = 
	// trigger_in_byte[4] = 
	// trigger_in_byte[5] = 
	// trigger_in_byte[6] = 
	// trigger_in_byte[7] = 
	active_trigger             ACTIVE_TRIGGER_INST
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),

	 .trigger_to_host          (trigger_out),
	 .trigger_to_device        (trigger_in_byte)
	
	);


	active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (transfer_1_start),
	 .transfer_received        (transfer_1_receive_in),
	 
	 .transfer_busy            (transfer_receive_busy),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (),//serial_receive_data_storage),
	 .transfer_to_device       (transfer_1_address)//()	
	);

	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (transfer_2_start),
	 .transfer_received        (transfer_2_receive_in),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_2_data_out),
	 .transfer_to_device       (transfer_2_data_in)	
	);
	
	active_transfer            ACTIVE_TRANSFER_INST_3
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (transfer_3_start),
	 .transfer_received        (transfer_3_receive_in),

	 .transfer_busy            (),
	
	 .uc_addr                  (3'h3),

	 .transfer_to_host         (),
	 .transfer_to_device       (transfer_3_control_register_number)	
	);

	active_transfer            ACTIVE_TRANSFER_INST_4
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 4*22 +: 22 ]),
	
	 .start_transfer           (transfer_4_start),
	 .transfer_received        (transfer_4_receive_in),
	
	 .transfer_busy            (),
	
	 .uc_addr                  (3'h4),

	 .transfer_to_host         (),
	 .transfer_to_device       (transfer_4_control_register_contents)	
	);
	
	
	active_block               BLOCK_TRANSFER_INST
	(
	 .uc_clk                   (CLK_50MHZ),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 5*22 +: 22 ]),
	
	 .start_transfer           (start_block_out),
	 .transfer_received        (block_in_rcv),
	 
	 .transfer_ready           (block_byte_ready),
	 .transfer_busy            (block_busy),

	 .ept_length               (),
	
	 .uc_addr                  (3'h1),
	 .uc_length                (MEMORY_TRANSFER_BLOCK_SIZE),

	 .transfer_to_host         (block_out_byte),
	 .transfer_to_device       ()
	
	);

	
endmodule
